package dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.google.common.base.Preconditions;
import com.mojang.datafixers.kinds.App;
import com.mojang.datafixers.kinds.Applicative;
import com.mojang.datafixers.util.Function5;
import com.mojang.serialization.Codec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemClass;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.GemBonus;
import dev.shadowsoffire.placebo.codec.PlaceboCodecs;
import java.util.Map;
import net.minecraft.ChatFormatting;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.network.chat.Style;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.enchantment.Enchantment;
import net.minecraftforge.registries.ForgeRegistries;

public class EnchantmentBonus extends GemBonus {
  protected final Enchantment ench;
  
  protected final boolean mustExist;
  
  protected final boolean global;
  
  protected final Map<LootRarity, Integer> values;
  
  public static Codec<dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.EnchantmentBonus> CODEC;
  
  static {
    CODEC = RecordCodecBuilder.create(inst -> inst.group(gemClass(), (App)ForgeRegistries.ENCHANTMENTS.getCodec().fieldOf("enchantment").forGetter(()), (App)PlaceboCodecs.nullableField((Codec)Codec.BOOL, "must_exist", Boolean.valueOf(false)).forGetter(()), (App)PlaceboCodecs.nullableField((Codec)Codec.BOOL, "global", Boolean.valueOf(false)).forGetter(()), (App)LootRarity.mapCodec(Codec.intRange(1, 127)).fieldOf("values").forGetter(())).apply((Applicative)inst, dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.EnchantmentBonus::new));
  }
  
  public EnchantmentBonus(GemClass gemClass, Enchantment ench, boolean mustExist, boolean global, Map<LootRarity, Integer> values) {
    super(Apotheosis.loc("enchantment"), gemClass);
    this.ench = ench;
    this.values = values;
    this.mustExist = mustExist;
    this.global = global;
  }
  
  public Component getSocketBonusTooltip(ItemStack gem, LootRarity rarity) {
    int level = ((Integer)this.values.get(rarity)).intValue();
    String desc = "bonus." + getId() + ".desc";
    if (this.global) {
      desc = desc + ".global";
    } else if (this.mustExist) {
      desc = desc + ".mustExist";
    } 
    MutableComponent enchName = Component.m_237115_(this.ench.m_44704_());
    Style style = this.ench.m_44700_(0).m_7383_();
    if (style.m_131135_() != null && style.m_131135_().m_131265_() != ChatFormatting.GRAY.m_126665_().intValue())
      enchName.m_130948_(style); 
    return (Component)Component.m_237110_(desc, new Object[] { Integer.valueOf(level), Component.m_237115_("misc.apotheosis.level" + ((level > 1) ? ".many" : "")), enchName }).m_130940_(ChatFormatting.GREEN);
  }
  
  public void getEnchantmentLevels(ItemStack gemStack, LootRarity rarity, Map<Enchantment, Integer> enchantments) {
    int level = ((Integer)this.values.get(rarity)).intValue();
    if (this.global) {
      for (Enchantment e : enchantments.keySet())
        enchantments.computeIfPresent(e, (key, val) -> Integer.valueOf((val.intValue() > 0) ? (val.intValue() + level) : 0)); 
    } else if (this.mustExist) {
      enchantments.computeIfPresent(this.ench, (key, val) -> Integer.valueOf((val.intValue() > 0) ? (val.intValue() + level) : 0));
    } else {
      enchantments.merge(this.ench, Integer.valueOf(level), Integer::sum);
    } 
  }
  
  public GemBonus validate() {
    Preconditions.checkNotNull(this.ench, "Invalid DamageReductionBonus with null type");
    Preconditions.checkNotNull(this.values, "Invalid DamageReductionBonus with null values");
    return this;
  }
  
  public boolean supports(LootRarity rarity) {
    return this.values.containsKey(rarity);
  }
  
  public int getNumberOfUUIDs() {
    return 0;
  }
  
  public Codec<? extends GemBonus> getCodec() {
    return (Codec)CODEC;
  }
}
